﻿// #######################################################################################################
// Program Name: CCR Client.exe
// Author: Brice Hilgemann
// Assistance: Professor Blahnik with TcpClient information, MSDN documentation on sockets
// Last Modified: 5/1/2010
// Status: Finalized (for now)
// Usage: Creates a client that connects to a Server and communicates asynchronously with this device by 
//        sending string information and also receives information from the server.
// #######################################################################################################

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Client
{
    public partial class Form1 : Form
    {
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     BEGINNING OF GLOBAL VARIABLES
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        
        Socket ClientSocket;                                 // Socket that will communicate with server
        IAsyncResult Result;                                 // Callback result from server socket
        AsyncCallback CallBack;                              // Actual callback used
        IPEndPoint address;                                  // Address of server
        
        byte[] dataholder = new byte[512];                   // Buffer for holding data
        Label[] turnouts = new Label[20];                    // For dynamic labels
        PictureBox[] turnoutimage = new PictureBox[20];      // For turnout images
        int turnouts_count = 6;                              // Number of turnouts on layout
        string train_id = "0";                               // Initial train_id


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     BEGINNING OF CODE AND EVENTS - DEFAULTS AND INITIALIZATION
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        public Form1(IPEndPoint connection)
        // ################################################################################################################
        //   Standard Form Initialization
        // ################################################################################################################
        {
            InitializeComponent();

            address = connection;
        }

        private void Form1_Shown(object sender, EventArgs e)
        // ################################################################################################################
        //   Default options set at startup when form is first shown
        // ################################################################################################################
        {
            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Client started");

            Form1.ActiveForm.StartPosition = FormStartPosition.CenterScreen;    // Default form settings
            Form1.ActiveForm.MaximizeBox = false;

            SetupTurnoutLabels();

            try
            {
                ClientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);         // Use IP and Port for ClientSocket
                ClientSocket.Connect(address);

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connected to Server");

                HandleDataFromServer();
            }
            catch
            {
                MessageBox.Show("Issue connecting - please try another IP and Port");
                Form1_FormClosing(null, null);
            }
        }

        private void SetupTurnoutLabels()
        // ################################################################################################################
        //   Hides turnout images and gets labels for turnouts
        // ################################################################################################################
        {
            pictureBox17.Visible = false;
            pictureBox18.Visible = false;
            pictureBox19.Visible = false;
            pictureBox20.Visible = false;
            pictureBox21.Visible = false;
            pictureBox22.Visible = false;

            turnouts[0] = label7;
            turnouts[1] = label8;
            turnouts[2] = label9;
            turnouts[3] = label10;
            turnouts[4] = label11;
            turnouts[5] = label12;

            turnoutimage[0] = pictureBox8;
            turnoutimage[1] = pictureBox9;
            turnoutimage[2] = pictureBox11;
            turnoutimage[3] = pictureBox10;
            turnoutimage[4] = pictureBox13;
            turnoutimage[5] = pictureBox12;
        }

        public void HandleDataFromServer()
        // ################################################################################################################
        //   Information received from Server
        //     - Bytes of data are now in the stream of which are sent from the server, callback was issued and this function
        //       is now called which handles the data
        // ################################################################################################################
        {
            try
            {
                if (CallBack == null)
                {
                    CallBack = new AsyncCallback(DataReceivedFromServer);
                }

                Result = ClientSocket.BeginReceive(dataholder, 0, dataholder.Length, SocketFlags.None, CallBack, ClientSocket);
            }
            catch
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connection issue");      // If something with connection fails
            }		
        }

        public void DataReceivedFromServer(IAsyncResult sync_result)
        // ################################################################################################################
        //   Process the data that was received from the server and decipher it to then perform certain actions
        // ################################################################################################################
		{
			try
			{
                Socket TempSocket = (Socket)sync_result.AsyncState;
                string[] data = new string[100];

                int received = TempSocket.EndReceive(sync_result);

                string received_info = Encoding.ASCII.GetString(dataholder, 0, received);    // Convert received to string
                
                data = received_info.Split(';');                                             //Check received_info for a message

                if (data[0] == "0")
                // ****************************************************************************************************************
                //   Disconnect received from the server
                //      [ COMMAND - 0 ]
                // ****************************************************************************************************************
                {
                    if (ClientSocket != null)
                    {
                        listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Closing Client");
                        ClientSocket.Close();             // Close connection with server
                        ClientSocket = null;
                    }

                    MessageBox.Show("There are currently no available trains on the track.  Try again later or add a train");

                    Application.Exit();
                }
                else if (data[0] == "1")
                // ****************************************************************************************************************
                //   Updates from the Server
                //      [ COMMAND - 1 ] ; [ TRAIN ID] ; [ SPEED ] ; [ DIRECTION ] ; [ LIGHT ] ; [ HORN ] ; [ BLOCKED ] ; [ TO_NEAREST ] ; [ NBR TOS ] ; [ TO# ST ]...
                // ****************************************************************************************************************
                {
                    if (data[3] == "Reverse")                                                           // Reverse
                    {
                        pictureBox14.Image = Properties.Resources.client_forward_off;
                        pictureBox15.Image = Properties.Resources.client_idle_off;
                        pictureBox16.Image = Properties.Resources.client_reverse;
                    }
                    else if (data[3] == "Forward")                                                       // Forward
                    {
                        pictureBox14.Image = Properties.Resources.client_forward; 
                        pictureBox15.Image = Properties.Resources.client_idle_off;
                        pictureBox16.Image = Properties.Resources.client_reverse_off;
                    }

                    if (data[6] == "false")
                    {
                        label3.Text = "Speed - " + data[2].ToString();        // Not Blocked
                        
                        if (data[2] == "0")
                        {
                            pictureBox14.Image = Properties.Resources.client_forward_off;               // Idle
                            pictureBox15.Image = Properties.Resources.client_idle;
                            pictureBox16.Image = Properties.Resources.client_reverse_off;
                        }
                    }
                    else
                    {
                        label3.Text = "** BLOCKED **";
                    }

                    if (data[4] == "false")  {  pictureBox6.Image = Properties.Resources.client_light_off;     }
                    else   {   pictureBox6.Image = Properties.Resources.client_light_on;  }        // Light

                    if (data[5] == "false")  {  pictureBox5.Image = Properties.Resources.client_sound_off;     }
                    else   {   pictureBox5.Image = Properties.Resources.client_sound_on;  }        // Horn

                    switch (data[7])                                                  // Closest Turnout
                    {
                        case "0":  {
                                pictureBox17.Visible = true; pictureBox18.Visible = false; pictureBox19.Visible = false;
                                pictureBox20.Visible = false; pictureBox21.Visible = false; pictureBox22.Visible = false;
                                break;   }
                        case "1":  {
                                pictureBox17.Visible = false; pictureBox18.Visible = true; pictureBox19.Visible = false;
                                pictureBox20.Visible = false; pictureBox21.Visible = false; pictureBox22.Visible = false;
                                break;   }
                        case "2":  {
                                pictureBox17.Visible = false; pictureBox18.Visible = false; pictureBox19.Visible = true;
                                pictureBox20.Visible = false; pictureBox21.Visible = false; pictureBox22.Visible = false;
                                break;   }
                        case "3":  {
                                pictureBox17.Visible = false; pictureBox18.Visible = false; pictureBox19.Visible = false;
                                pictureBox20.Visible = true; pictureBox21.Visible = false; pictureBox22.Visible = false;
                                break;   }
                        case "4":  {
                                pictureBox17.Visible = false; pictureBox18.Visible = false; pictureBox19.Visible = false;
                                pictureBox20.Visible = false; pictureBox21.Visible = true; pictureBox22.Visible = false;
                                break;   }
                        case "5":  {
                                pictureBox17.Visible = false; pictureBox18.Visible = false; pictureBox19.Visible = false;
                                pictureBox20.Visible = false; pictureBox21.Visible = false; pictureBox22.Visible = true;
                                break;   }
                        default:   {
                                pictureBox17.Visible = false; pictureBox18.Visible = false; pictureBox19.Visible = false;
                                pictureBox20.Visible = false; pictureBox21.Visible = false; pictureBox22.Visible = false;
                                break;   }
                    }

                    for (int i = 0; i < Convert.ToInt32(data[8]); i++)
                    {
                        if (data[9 + i] == "top")
                        {
                            turnoutimage[i].Image = Properties.Resources.client_turnout_red;
                        }
                        else
                        {
                            turnoutimage[i].Image = Properties.Resources.client_turnout_green;
                        }
                    }
                }
                else if(data[0] == "2")
                // ****************************************************************************************************************
                //   Disconnect removed from the server
                //      [ COMMAND - 2 ]
                // ****************************************************************************************************************
                {
                    if (ClientSocket != null)
                    {
                        listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Closing Client");
                        ClientSocket.Close();             // Close connection with server
                        ClientSocket = null;
                    }

                    MessageBox.Show("Train has been removed by server.  Please add a new train or thanks for playing!");

                    Application.Exit();
                }
                else if (data[0] == "3")
                // ****************************************************************************************************************
                //   Receive the train_id and turnout numbers from the server for filling the labels
                //      [ COMMAND - 3 ] ; [Train ID ] ; [ Nbr TOs ] ; [ TO Nbr ]...
                // ****************************************************************************************************************
                {
                    label5.Text = data[1];                               // Train ID
                    train_id = data[1];

                    if (Convert.ToInt32(data[2]) > turnouts_count)
                    {
                        MessageBox.Show("Issue with turnouts not matching current client view.");
                        Form1_FormClosing(null, null);
                    }

                    for (int i = 0; i < Convert.ToInt32(data[2]); i++)
                    {
                        turnouts[i].Text = data[3 + i];
                    }
                }

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - From Server " + received_info);    //Output string received

				HandleDataFromServer();
			}
            catch
            {
                //Server has disconnected
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server has disconnected");
                MessageBox.Show("Server has disconnected - Program will now close");

                Form1_FormClosing(null, null);
            }
		}	

        private void SendtoServer(string command)
        // ################################################################################################################
        //   Takes information from command and sends it to the server through the stream
        // ################################################################################################################
        {
            try
            {
                byte[] data = Encoding.ASCII.GetBytes(command);      // Convert string information to bytes to send through stream
                ClientSocket.Send(data);                             // Write data to stream to send to server
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent " + command + " to Server");
            }
            catch
            {
                MessageBox.Show(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Send issue");    // If something with connection fails
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        // ################################################################################################################
        //   Handles closing connections before the application closes
        // ################################################################################################################
        {
            if (ClientSocket != null)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Closing Client");
                ClientSocket.Close();             // Close connection with server
                ClientSocket = null;
            }

            Application.Exit();
        }


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     SPECIFIC TRAIN ACTIONS OR EVENTS TO CHANGE ON RAILROAD
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        private void pictureBox2_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Move the train 'forward'
        //       [ COMMAND - 1 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("1;" + train_id);
        }

        private void pictureBox4_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Move the train 'backward'
        //       [ COMMAND - 2 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("2;" + train_id);
        }

        private void pictureBox3_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Stop the train
        //       [ COMMAND - 3 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("3;" + train_id);
        }

        private void pictureBox15_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Change the train to 'idle'
        // ################################################################################################################
        {
            pictureBox3_Click(sender, e);
        }

        private void pictureBox14_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Change the train to 'forward'
        //       [ COMMAND - 4 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("4;" + train_id);
        }

        private void pictureBox16_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Change the train to 'backward'
        //       [ COMMAND - 5 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("5;" + train_id);
        }

        private void pictureBox6_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Toggle the train light
        //       [ COMMAND - 9 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("9;" + train_id);
        }

        private void pictureBox5_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Send command to the railroad - Toggle the train horn
        //       [ COMMAND - 10 ] ; [ DATA - Train_ID ]
        // ################################################################################################################
        {
            SendtoServer("10;" + train_id);
        }


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     CHANGE DIFFERENT TURNOUTS - TOGGLE AND SHOW CLOSEST
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        private void pictureBox8_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the first turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;0");
        }

        private void pictureBox9_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the second turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;1");
        }

        private void pictureBox11_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the third turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;2");
        }

        private void pictureBox10_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the fourth turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;3");
        }

        private void pictureBox13_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the fifth turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;4");
        }

        private void pictureBox12_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Toggle the sixth turnout on the track
        //       [ COMMAND - 6 ] ; [ DATA - Turnout_ID ]
        // ################################################################################################################
        {
            SendtoServer("6;5");
        }


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     MENUSTRIP ITEM EVENTS - DO SPECIFIC ACTIONS
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        private void disconnectToolStripMenuItem_Click(object sender, EventArgs e)
        // ################################################################################################################
        //   Disconnect button clicked - close form
        // ################################################################################################################
        {
            Form1_FormClosing(null, null);
        }


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//     BEGINNING OF CODE AND EVENTS - DEFAULTS AND INITIALIZATION
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        private void pictureBox2_MouseHover(object sender, EventArgs e)
        // ################################################################################################################
        //   Hovering over the 'Up' button - swap image
        // ################################################################################################################
        { pictureBox2.Image = Properties.Resources.client_up_over; }

        private void pictureBox3_MouseHover(object sender, EventArgs e)
        // ################################################################################################################
        //   Hovering over the 'Stop' button - swap image
        // ################################################################################################################
        { pictureBox3.Image = Properties.Resources.client_stop_over; }

        private void pictureBox4_MouseHover(object sender, EventArgs e)
        // ################################################################################################################
        //   Hovering over the 'Down' button - swap image
        // ################################################################################################################
        { pictureBox4.Image = Properties.Resources.client_down_over; }

        private void pictureBox2_MouseLeave(object sender, EventArgs e)
        // ################################################################################################################
        //   Leaving the 'Up' button - swap image back to usual
        // ################################################################################################################
        { pictureBox2.Image = Properties.Resources.client_up; }

        private void pictureBox3_MouseLeave(object sender, EventArgs e)
        // ################################################################################################################
        //   Leaving the 'Stop' button - swap image back to usual
        // ################################################################################################################
        { pictureBox3.Image = Properties.Resources.client_stop; }

        private void pictureBox4_MouseLeave(object sender, EventArgs e)
        // ################################################################################################################
        //   Leaving the 'Down' button - swap image back to usual
        // ################################################################################################################
        { pictureBox4.Image = Properties.Resources.client_down; }
    }
}
